@@ -13,7 +13,7 @@ import attr
 import requests
 import tenacity
 
-from . import config, Line, t
+from . import config, line, t
 
 
 @attr.s(auto_attribs=True)
@@ -22,14 +22,24 @@ class InputContent:
     date: t.Optional[datetime.date]
 
     @property
-    def lines(self) -> t.List[Line.Line]:
-        return [Line.Line(i, line) for i, line in enumerate(self.rawLines, 1)]
+    def lines(self) -> t.List[line.Line]:
+        return [line.Line(lineNo, text) for lineNo, text in enumerate(self.rawLines, 1)]
 
     @property
     def content(self) -> str:
         return "".join(self.rawLines)
 
 
+def inputFromName(sourceName: str, **kwargs) -> InputSource:
+    if sourceName == "-":
+        return StdinInputSource(sourceName)
+    if sourceName.startswith("https:"):
+        return UrlInputSource(sourceName)
+    if not sourceName.endswith((".bs", ".src.html")) and os.path.exists(sourceName) and tarfile.is_tarfile(sourceName):
+        return TarInputSource(sourceName, **kwargs)
+    return FileInputSource(sourceName, **kwargs)
+
+
 class InputSource:
     """Represents a thing that can produce specification input text.
 
@@ -38,24 +48,6 @@ class InputSource:
     manager for temporarily switching to the directory of a file InputSource.
     """
 
-    def __new__(cls, sourceName: str, **kwargs):
-        """Dispatches to the right subclass."""
-        if cls != InputSource:
-            # Only take control of calls to InputSource(...) itself.
-            return super().__new__(cls)
-
-        if sourceName == "-":
-            return StdinInputSource(sourceName)
-        if sourceName.startswith("https:"):
-            return UrlInputSource(sourceName)
-        if (
-            not sourceName.endswith((".bs", ".src.html"))
-            and os.path.exists(sourceName)
-            and tarfile.is_tarfile(sourceName)
-        ):
-            return TarInputSource(sourceName, **kwargs)
-        return FileInputSource(sourceName, **kwargs)
-
     @abstractmethod
     def __str__(self) -> str:
         pass
