@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import copy
 import os
 import re
@@ -5,7 +7,7 @@ import subprocess
 from collections import OrderedDict, defaultdict
 from datetime import datetime
 
-from . import biblio, conditional, config, dfnpanels, h, messages as m, refs as r, retrieve
+from . import biblio, conditional, config, dfnpanels, h, messages as m, refs as r, retrieve, t
 from .DefaultOrderedDict import DefaultOrderedDict
 
 
@@ -235,7 +237,7 @@ def removeUnwantedBoilerplate(doc):
                 h.removeNode(el)
 
 
-def addAnnotations(doc):
+def addAnnotations(doc: t.SpecT) -> None:
     if doc.md.vshortname in doc.testSuites:
         html = retrieve.retrieveBoilerplateFile(doc, "annotations")
         el = boilerplateFromHtml(doc, html)
@@ -349,7 +351,7 @@ def addIndexOfLocallyDefinedTerms(doc, container):
     h.appendChild(container, indexHTML, allowEmpty=True)
 
 
-def disambiguator(ref, types, specs):
+def disambiguator(ref: r.RefWrapper, types: t.Optional[t.Set[str]], specs: t.Optional[t.Set[str]]) -> str:
     disambInfo = []
     if types is None or len(types) > 1:
         disambInfo.append(ref.type)
@@ -374,16 +376,16 @@ def addExplicitIndexes(doc):
             continue
 
         if el.get("type"):
-            types = {x.strip() for x in el.get("type").split(",")}
-            for t in types:
-                if t not in config.dfnTypes:
+            elTypes = {x.strip() for x in el.get("type").split(",")}
+            for elType in elTypes:
+                if elType not in config.dfnTypes:
                     m.die(
-                        f"Unknown type value '{t}' on {h.outerHTML(el)}",
+                        f"Unknown type value '{elType}' on {h.outerHTML(el)}",
                         el=el,
                     )
-                    types.remove(t)
+                    elTypes.remove(elType)
         else:
-            types = None
+            elTypes = None
 
         if el.get("data-link-spec"):
             # Yes, this is dumb. Accidental over-firing of a shortcut attribute. >_<
@@ -423,7 +425,7 @@ def addExplicitIndexes(doc):
                 continue
             if specs is not None and ref.spec not in specs:
                 continue
-            if types is not None and ref.type not in types:
+            if elTypes is not None and ref.type not in elTypes:
                 continue
             if fors is not None and not (set(ref.for_) & fors):
                 continue
@@ -433,9 +435,9 @@ def addExplicitIndexes(doc):
         # ensuring no duplicate disambiguators.
         refsFromText = defaultdict(list)
         for ref in possibleRefs:
-            refDisambiguator = disambiguator(ref, types, specs)
+            refDisambiguator = disambiguator(ref, elTypes, specs)
             for i, existingRef in enumerate(refsFromText[ref.text]):
-                if disambiguator(existingRef, types, specs) != refDisambiguator:
+                if disambiguator(existingRef, elTypes, specs) != refDisambiguator:
                     continue
                 # Whoops, found an identical entry.
                 if existingRef.status != ref.status:
@@ -473,7 +475,7 @@ def addExplicitIndexes(doc):
             refs = r.utils.filterOldVersions(refs)
             if refs:
                 filteredRefs[ttf[0]].extend(
-                    {"url": ref.url, "disambiguator": disambiguator(ref, types, specs)} for ref in refs
+                    {"url": ref.url, "disambiguator": disambiguator(ref, elTypes, specs)} for ref in refs
                 )
 
         h.appendChild(el, htmlFromIndexTerms(filteredRefs), allowEmpty=True)
@@ -868,8 +870,8 @@ def addTOCSection(doc):
         h.removeNode(el)
 
 
-def addSpecMetadataSection(doc):
-    def printEditor(editor):
+def addSpecMetadataSection(doc: t.SpecT) -> None:
+    def printEditor(editor) -> t.Optional[t.ElementT]:
         dd = h.E.dd({"class": "editor p-author h-card vcard"})
         if editor["w3cid"]:
             dd.attrib["data-editor-id"] = editor["w3cid"]
@@ -911,7 +913,7 @@ def addSpecMetadataSection(doc):
             )
         return dd
 
-    def printTranslation(tr):
+    def printTranslation(tr) -> t.Optional[t.ElementT]:
         lang = tr["lang-code"]
         # canonicalize the lang-code structure
         lang = lang.lower().replace("_", "-")
@@ -921,12 +923,12 @@ def addSpecMetadataSection(doc):
         missingInfo = False
         if name is None:
             if lang in doc.languages:
-                name = doc.languages[lang]["name"]
+                name = doc.languages[lang].name
             else:
                 missingInfo = True
         if nativeName is None:
             if lang in doc.languages:
-                nativeName = doc.languages[lang]["native-name"]
+                nativeName = doc.languages[lang].nativeName
             else:
                 missingInfo = True
         if missingInfo:
@@ -945,7 +947,7 @@ def addSpecMetadataSection(doc):
             return h.E.a({"href": url, "hreflang": lang, "rel": "alternate", "title": lang}, name)
         return h.E.a({"href": url, "hreflang": lang, "rel": "alternate"}, lang)
 
-    def printPreviousVersion(v):
+    def printPreviousVersion(v) -> t.Optional[t.ElementT]:
         if v["type"] == "url":
             return h.E.a({"href": v["value"], "rel": "prev"}, v["value"])
         if v["type"] == "from-biblio":
@@ -957,7 +959,7 @@ def addSpecMetadataSection(doc):
             m.die(
                 f"While trying to generate a Previous Version line, couldn't find a dated biblio reference for {key}."
             )
-            return
+            return None
         return h.E.a({"href": dated.url, "rel": "prev"}, dated.url)
 
     md = DefaultOrderedDict(list)
@@ -1006,8 +1008,8 @@ def addSpecMetadataSection(doc):
         md["Implementation Report"].append(h.E.a({"href": doc.md.implementationReport}, doc.md.implementationReport))
     if doc.md.testSuite is not None:
         md["Test Suite"].append(h.E.a({"href": doc.md.testSuite}, doc.md.testSuite))
-    elif (doc.md.vshortname in doc.testSuites) and (doc.testSuites[doc.md.vshortname]["url"] is not None):
-        url = doc.testSuites[doc.md.vshortname]["url"]
+    elif (doc.md.vshortname in doc.testSuites) and (doc.testSuites[doc.md.vshortname].url is not None):
+        url = doc.testSuites[doc.md.vshortname].url
         md["Test Suite"].append(h.E.a({"href": url}, url))
     if doc.md.issues:
         if doc.md.TR:
