@@ -39,8 +39,7 @@ from .main import (
     doEvery,
 )
 from .Nil import Nil
-from .printjson import printjson
-from .retrieve import DataFileRequester, defaultRequester, retrieveBoilerplateFile
+from .printjson import printjson, getjson
 from .status import (
     shortToLongStatus,
     snapshotStatuses,
