@@ -3,7 +3,10 @@
 from __future__ import annotations
 
 import sys
+from collections import defaultdict
+
 from typing import (
+    AbstractSet,
     Any,
     cast,
     Callable,
@@ -19,15 +22,25 @@ from typing import (
     overload,
     Sequence,
     Set,
+    TextIO,
     Tuple,
     TYPE_CHECKING,
     TypeVar,
     Union,
 )
 
-from lxml import etree
-from typing_extensions import Literal, TypeAlias
+if sys.version_info >= (3, 8):
+    from typing import Literal, Protocol
+else:
+    from typing_extensions import Literal, Protocol
 
+if sys.version_info >= (3, 10):
+    from typing import TypeAlias
+else:
+    from typing_extensions import TypeAlias
+
+
+from lxml import etree  # pylint: disable=wrong-import-position
 
 ElementT: TypeAlias = etree._Element  # pylint: disable=protected-access
 DocumentT: TypeAlias = etree._ElementTree  # pylint: disable=protected-access
@@ -43,10 +56,10 @@ if TYPE_CHECKING:
         from .Spec import Spec  # pylint: disable=cyclic-import
     SpecT = Spec
 
-if TYPE_CHECKING:
     from . import biblio  # pylint: disable=cyclic-import
 
     BiblioStorageT: TypeAlias = DefaultDict[str, List[biblio.BiblioEntry]]
 
-if TYPE_CHECKING:
     FillContainersT: TypeAlias = DefaultDict[str, List[ElementT]]
+
+    LinkDefaultsT: TypeAlias = defaultdict[str, list[tuple[str, str, str | None, str | None]]]
